@@ -1,4 +1,4 @@
-# Copyright (C) 2001-2014 by the Free Software Foundation, Inc.
+# Copyright (C) 2001-2018 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -50,35 +50,16 @@ class ModeratedMemberPost(Hold.ModeratedPost):
 def process(mlist, msg, msgdata):
     if msgdata.get('approved'):
         return
-    # Before anything else, check DMARC if necessary.
-    msgdata['from_is_list'] = 0
-    dn, addr = parseaddr(msg.get('from'))
-    if addr and mlist.dmarc_moderation_action > 0:
-        if Utils.IsDMARCProhibited(mlist, addr):
-            # Note that for dmarc_moderation_action, 0 = Accept, 
-            #    1 = Munge, 2 = Wrap, 3 = Reject, 4 = Discard
-            if mlist.dmarc_moderation_action == 1:
-                msgdata['from_is_list'] = 1
-            elif mlist.dmarc_moderation_action == 2:
-                msgdata['from_is_list'] = 2
-            elif mlist.dmarc_moderation_action == 3:
-                # Reject
-                text = mlist.dmarc_moderation_notice
-                if text:
-                    text = Utils.wrap(text)
-                else:
-                    text = Utils.wrap(_(
-"""You are not allowed to post to this mailing list From: a domain which
-publishes a DMARC policy of reject or quarantine, and your message has been
-automatically rejected.  If you think that your messages are being rejected in
-error, contact the mailing list owner at %(listowner)s."""))
-                raise Errors.RejectMessage, text
-            elif mlist.dmarc_moderation_action == 4:
-                raise Errors.DiscardMessage
-    # Then, is the poster a member or not?
+    # Is the poster a member or not?
     for sender in msg.get_senders():
         if mlist.isMember(sender):
             break
+        for sender in Utils.check_eq_domains(sender,
+                          mlist.equivalent_domains):
+            if mlist.isMember(sender):
+                break
+        if mlist.isMember(sender):
+            break
     else:
         sender = None
     if sender:
@@ -116,15 +97,27 @@ error, contact the mailing list owner at %(listowner)s."""))
         sender = msg.get_sender()
     # From here on out, we're dealing with non-members.
     listname = mlist.internal_name()
-    if matches_p(sender, mlist.accept_these_nonmembers, listname):
+    if mlist.GetPattern(sender,
+                        mlist.accept_these_nonmembers,
+                        at_list='accept_these_nonmembers'
+                       ):
         return
-    if matches_p(sender, mlist.hold_these_nonmembers, listname):
+    if mlist.GetPattern(sender,
+                        mlist.hold_these_nonmembers,
+                        at_list='hold_these_nonmembers'
+                       ):
         Hold.hold_for_approval(mlist, msg, msgdata, Hold.NonMemberPost)
         # No return
-    if matches_p(sender, mlist.reject_these_nonmembers, listname):
+    if mlist.GetPattern(sender,
+                        mlist.reject_these_nonmembers,
+                        at_list='reject_these_nonmembers'
+                       ):
         do_reject(mlist)
         # No return
-    if matches_p(sender, mlist.discard_these_nonmembers, listname):
+    if mlist.GetPattern(sender,
+                        mlist.discard_these_nonmembers,
+                        at_list='discard_these_nonmembers'
+                       ):
         do_discard(mlist, msg)
         # No return
     # Okay, so the sender wasn't specified explicitly by any of the non-member
@@ -142,43 +135,6 @@ error, contact the mailing list owner at %(listowner)s."""))
         do_discard(mlist, msg)
 
 
-
-def matches_p(sender, nonmembers, listname):
-    # First strip out all the regular expressions and listnames
-    plainaddrs = [addr for addr in nonmembers if not (addr.startswith('^')
-                                                 or addr.startswith('@'))]
-    addrdict = Utils.List2Dict(plainaddrs, foldcase=1)
-    if addrdict.has_key(sender):
-        return 1
-    # Now do the regular expression matches
-    for are in nonmembers:
-        if are.startswith('^'):
-            try:
-                cre = re.compile(are, re.IGNORECASE)
-            except re.error:
-                continue
-            if cre.search(sender):
-                return 1
-        elif are.startswith('@'):
-            # XXX Needs to be reviewed for list@domain names.
-            try:
-                mname = are[1:].lower().strip()
-                if mname == listname:
-                    # don't reference your own list
-                    syslog('error',
-                        '*_these_nonmembers in %s references own list',
-                        listname)
-                else:
-                    mother = MailList(mname, lock=0)
-                    if mother.isMember(sender):
-                        return 1
-            except Errors.MMUnknownListError:
-                syslog('error',
-                  '*_these_nonmembers in %s references non-existent list %s',
-                  listname, mname)
-    return 0
-
-
 
 def do_reject(mlist):
     listowner = mlist.GetOwnerEmail()
@@ -187,9 +143,10 @@ def do_reject(mlist):
               Utils.wrap(_(mlist.nonmember_rejection_notice))
     else:
         raise Errors.RejectMessage, Utils.wrap(_("""\
-You are not allowed to post to this mailing list, and your message has been
-automatically rejected.  If you think that your messages are being rejected in
-error, contact the mailing list owner at %(listowner)s."""))
+Your message has been rejected, probably because you are not subscribed to the
+mailing list and the list's policy is to prohibit non-members from posting to
+it.  If you think that your messages are being rejected in error, contact the
+mailing list owner at %(listowner)s."""))
 
 
 
