@@ -1,4 +1,4 @@
-# Copyright (C) 2000,2001,2002 by the Free Software Foundation, Inc.
+# Copyright (C) 2000-2018 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,13 +12,14 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 """Archive queue runner."""
 
 import time
 from email.Utils import parsedate_tz, mktime_tz, formatdate
 
+from Mailman import i18n
 from Mailman import mm_cfg
 from Mailman import LockFile
 from Mailman.Queue.Runner import Runner
@@ -49,7 +50,7 @@ class ArchRunner(Runner):
                 elif abs(now - mktime_tz(tup)) > \
                          mm_cfg.ARCHIVER_ALLOWABLE_SANE_DATE_SKEW:
                     clobber = 1
-            except ValueError:
+            except (ValueError, OverflowError):
                 # The likely cause of this is that the year in the Date: field
                 # is horribly incorrect, e.g. (from SF bug # 571634):
                 # Date: Tue, 18 Jun 0102 05:12:09 +0500
@@ -70,6 +71,9 @@ class ArchRunner(Runner):
             # oh well, try again later
             return 1
         try:
+            # Archiving should be done in the list's preferred language, not
+            # the sender's language.
+            i18n.set_language(mlist.preferred_language)
             mlist.ArchiveMail(msg)
             mlist.Save()
         finally:
